package org.jboss.soa.qa.jmsbindingrollback;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.switchyard.component.bean.Service;

import javax.naming.InitialContext;
import javax.transaction.Transaction;
import javax.transaction.TransactionManager;

@Service(RollbackComponent.class)
public class RollbackComponentBean implements RollbackComponent {

	private static final Logger logger = LoggerFactory.getLogger(RollbackComponentBean.class);
	private static final String JNDI_TRANSACTION_MANAGER = "java:jboss/TransactionManager";

	private int rollbackCounter = 0;

	@Override
	public void rollback(String command) {
		Transaction t = getCurrentTransaction();
		logger.info("Processing rollback for command: {} and transaction: {}", command, t.toString());
		try {
			if (++rollbackCounter % 3 != 0) {
				t.setRollbackOnly();
				logger.info("Marked transaction to rollback!");
			} else {
				logger.info("Rollbacks completed - will be committed");
			}
		} catch (Exception e) {
			logger.error("Failed to rollback transaction", e);
		}
	}

	private Transaction getCurrentTransaction() {
		try {
			TransactionManager tm = (TransactionManager) new InitialContext().lookup(JNDI_TRANSACTION_MANAGER);
			Transaction t =  tm.getTransaction();
			if (t == null) {
				logger.error("No active transaction");
			}
			return t;
		} catch (Exception e) {
			logger.error("Failed to get current transaction");
			return null;
		}
	}
}
