package org.jboss.soa.qa.jmsbindingrollback;

import org.jboss.arquillian.container.test.api.Deployment;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.shrinkwrap.api.Archive;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.importer.ZipImporter;
import org.jboss.shrinkwrap.api.spec.JavaArchive;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;

import javax.annotation.Resource;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;
import java.io.File;
import java.util.zip.ZipFile;

@RunWith(Arquillian.class)
public class JMSBindingRollbackTest {

	private static final String JAR_FILE = "target/jms-binding-rollback-issue.jar";

	@Deployment
	public static Archive<?> createTestArchive() throws Exception {
		File artifact = new File(JAR_FILE);
		try {
			return ShrinkWrap.create(ZipImporter.class, artifact.getName()).importFrom(new ZipFile(artifact)).as(JavaArchive.class);
		} catch (Exception e) {
			throw new RuntimeException(JAR_FILE + " not found. Run 'mvn package'", e);
		}
	}

	@Resource(mappedName = "/ConnectionFactory")
	private ConnectionFactory connectionFactory;

	@Resource(mappedName = "JmsBindingIssueInQ")
	private Destination queue;

	@Test
	public void testJPABindingTransaction() throws Exception {
		Connection connection = connectionFactory.createConnection();
		try {
			Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
			MessageProducer producer = session.createProducer(queue);
			TextMessage message = session.createTextMessage();
			message.setText("test");
			producer.send(message);
			session.close();

			Thread.sleep(20000); // wait for rollbacks and final commit

			session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
			MessageConsumer consumer = session.createConsumer(queue);
			connection.start();
			Assert.assertNull("Message was not consumed: " + queue.toString(), consumer.receive(3000));
		} finally {
			connection.close();
		}
	}
}
